% $Id: translate.sl,v 1.1 2005/02/02 20:14:56 robin Exp $
%
% A dict.leo.org mode for JED.
% 
% To use simply bind the translate() function to some key.
% 
% Robin Sommer <sommer@in.tum.de>
  
require("bufutils");
require("txtutils");
require("datutils");

static variable TranslateBuf = "*dict.leo.org*";
static variable TranslateCommand = "translate";
static variable KeyBindings_Help = "Enter:Select Space:Lookup Backspace:Back q:Quit";

static variable mode = "translate";
implements(mode);

static variable OldBuf = "";
static variable LastLookup = "";
static variable History = String_Type[0];

% Create the keymap
!if (keymap_p(mode)) {
   make_keymap(mode);
   definekey("translate_select", " ", mode);
   definekey("translate_lookup", "^M", mode);
   definekey("translate_back", Key_BS, mode);
   definekey("translate_close", "q", mode);
   definekey("translate_next_word", Key_Right, mode);
   definekey("translate_prev_word", Key_Left, mode);
   definekey("translate_next_line", Key_Down, mode);
   definekey("translate_prev_line", Key_Up, mode);
}

% FIXME: Why can't we call this translate_mode()?
define init_translate_mode()
{   
   set_mode(mode, 0);
   use_keymap(mode);
   message(KeyBindings_Help);
}

public define translate()
{
   variable what = "";

   if (_NARGS)
     what = ();

   if (what == "" and is_visible_mark())
        what = get_word();
   
   if (what == "") {
        what = read_mini("Translate:", "", "");
        !if ( strlen(what) ) return;
     }
   
   if ( OldBuf == "" )
     OldBuf = whatbuf();

   setbuf(TranslateBuf);
   set_readonly(0);
   erase_buffer();

   set_status_line(TranslateBuf + "      Looking up " + what + "... (%p)", 0);
   update(1);
   
   popup_buffer(whatbuf());
   init_translate_mode();

   set_prefix_argument(1);     % insert result in current buffer
   do_shell_cmd(TranslateCommand + " '" + what + "'");
   LastLookup = what;
   
   set_readonly(1);
   set_buffer_modified_flag(0);
   fit_window(.6);
}

public define translate_lookup()
{
   variable word = get_word();
   History = array_append(History, LastLookup);
   translate(word);
}

public define translate_back()
{
   if ( length(History) == 0 )
     message("Nothing to go back.");
   else {
      variable last = History[-1];
      History = array_delete(History, -1);
      translate(last);
   }
}

public define translate_close()
{
   History = String_Type[0];
   close_buffer(TranslateBuf);
   sw2buf(OldBuf);
   OldBuf = "";
}

public define translate_select()
{
   variable word = get_word();
   translate_close();
   insert(word);
}

public define translate_next_word()
{
   skip_word_chars();
   skip_non_word_chars();
}

public define translate_prev_word()
{
   bskip_word_chars();
   bskip_non_word_chars();
   left(1);
   bskip_word_chars();
}

public define translate_next_line()
{
   variable c = what_column();
   down(1);
   goto_column(c);
}

public define translate_prev_line()
{
   variable c = what_column();
   up(1);
   goto_column(c);
}

provide(mode);

